# Load the data
nlsy_math_wide<-read.table("C:/GRE2016/Data/nlsy_math_wide_R.dat", na.string='.')


# Give the variable names
names(nlsy_math_wide)<-c('id','female','lb_wght','anti_k1',
                         'math2','math3','math4','math5','math6','math7','math8',
                         'age2','age3','age4','age5','age6','age7','age8',
                         'men2','men3','men4','men5','men6','men7','men8',
                         'spring2','spring3','spring4','spring5','spring6','spring7','spring8',
                         'anti2','anti3','anti4','anti5','anti6','anti7','anti8')

mathdata_N <- subset(nlsy_math_wide,lb_wght==0)
mathdata_L <- subset(nlsy_math_wide,lb_wght==1)

library(OpenMx)

################# Invariance Model ################# 
mg1.math.omx <- mxModel("Multiple Group Linear Growth Invariance Model, Path Specification",
mxModel("group1",
type="RAM",
mxData(observed=mathdata_N, type="raw"),
manifestVars=c("math2","math3","math4","math5","math6","math7","math8"),
latentVars=c("eta_1","eta_2"),

# residual variance paths
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=2, free=TRUE, values=60, labels="th"),

# latent variable variances and covariance paths 
mxPath(from=c("eta_1","eta_2"), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(1,0,1), labels=c("psi_11","psi_21","psi_22")),

# Factor Loadings
mxPath(from="eta_1", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=1),

mxPath(from="eta_2", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

# means and intercepts
mxPath(from="one", to=c("eta_1","eta_2"),
 arrows=1, free=TRUE, values=c(100, 15), labels=c("alpha_1","alpha_2"))

),
 # close Normal Birthweight Model
 
mxModel("group2",
type="RAM",
mxData(
observed=mathdata_L,
type="raw"),
manifestVars=c("math2","math3","math4","math5","math6","math7","math8"),
latentVars=c("eta_1","eta_2"),

# residual variance paths
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=2, free=TRUE, values=60, labels="th"),

# latent variable variances and covariance paths 
mxPath(from=c("eta_1","eta_2"), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(1,0,1), labels=c("psi_11","psi_21","psi_22")),

# Factor Loadings
mxPath(from="eta_1", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=1),

mxPath(from="eta_2", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

# means and intercepts
mxPath(from="one", to=c("eta_1","eta_2"), arrows=1, free=TRUE,
 values=c(100, 15), labels=c("alpha_1","alpha_2"))

),
#Close Low Birthweight Model

mxAlgebra(group1.objective + group2.objective, name="mg_objective"),
mxFitFunctionAlgebra("mg_objective")
)

mg1.math.fit <- mxRun(mg1.math.omx)

summary(mg1.math.fit)


#free parameters:
#     name   matrix   row   col   Estimate  Std.Error lbound ubound
#1      th group1.S math2 math2 36.2296341 1.84376925              
#2  psi_11 group1.S eta_1 eta_1 64.5632799 5.47402771              
#3  psi_21 group1.S eta_1 eta_2 -0.1816808 1.07683449              
#4  psi_22 group1.S eta_2 eta_2  0.7326058 0.31666707              
#5 alpha_1 group1.M     1 eta_1 35.2673403 0.35540319              
#6 alpha_2 group1.M     1 eta_2  4.3393369 0.08829877              
#
#observed statistics:  2221 
#estimated parameters:  6 
#degrees of freedom:  2215 
#-2 log likelihood:  15937.39 
#saturated -2 log likelihood:  NA 
#number of observations:  933 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC: 11507.3857           15949.39                   NA
#BIC:   790.3181           15978.42             15959.36
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-06-02 12:26:00 
#frontend time: 0.1529999 secs 
#backend time: 0.6040001 secs 
#independent submodels time: 0 secs 
#wall clock time: 0.757 secs 
#cpu time: 0.757 secs 
#OpenMx version number: 999.0.0-3473 










################# Means Model ################# 
mg2.math.omx <- mxModel("Multiple Group Growth Model, Means, Covariances, Residuals, Path Specification",
mxModel("group1", type="RAM",
mxData(observed=mathdata_N, type="raw"),
manifestVars=c("math2","math3","math4","math5","math6","math7","math8"), 
latentVars=c("eta_1","eta_2"),

# residual variance paths
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=2, free=TRUE, values=60, labels="th"),

# latent variable variances and covariance paths 
mxPath(from=c("eta_1","eta_2"), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(1,0,1), labels=c("psi_11","psi_21","psi_22")),

# Factor Loadings
mxPath(from="eta_1", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=1),

mxPath(from="eta_2", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

# means and intercepts 
mxPath(from="one", to=c("eta_1","eta_2"),
 arrows=1, free=c(TRUE,TRUE), values=c(100, 15), labels=c("alpha_1_N","alpha_2_N"))

),  # close Normal Birthweight Model   

mxModel("group2", type="RAM",
mxData(observed=mathdata_L, type="raw"),
manifestVars=c("math2","math3","math4","math5","math6","math7","math8"),
latentVars=c("eta_1","eta_2"),

# residual variance paths
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=2, free=TRUE, values=60, labels="th"),

# latent variable variances and covariance paths 
mxPath(from=c("eta_1","eta_2"), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(1,0,1), labels=c("psi_11","psi_21","psi_22")),

# Factor Loadings
mxPath(from="eta_1", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=1),

mxPath(from="eta_2",  to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

# means and intercepts 
mxPath(from="one", to=c("eta_1","eta_2"),
 arrows=1, free=c(TRUE,TRUE), values=c(100, 15), labels=c("alpha_1_L","alpha_2_L"))

), #Close Low Birthweight Model

mxAlgebra(group1.objective + group2.objective, name="mg_objective"),
mxFitFunctionAlgebra("mg_objective")

) #Close Model

mg2.math.fit <- mxRun(mg2.math.omx)

summary(mg2.math.fit)

#free parameters:
#free parameters:
#       name   matrix   row   col    Estimate  Std.Error lbound ubound
#1        th group1.S math2 math2 36.32065031 1.84059930              
#2    psi_11 group1.S eta_1 eta_1 63.70590467 5.39562218              
#3    psi_21 group1.S eta_1 eta_2 -0.03549724 1.05174657              
#4    psi_22 group1.S eta_2 eta_2  0.69898080 0.31147782              
#5 alpha_1_N group1.M     1 eta_1 35.48753858 0.36935150              
#6 alpha_2_N group1.M     1 eta_2  4.29228574 0.09150566              
#7 alpha_1_L group2.M     1 eta_1 32.73334782 1.24390159              
#8 alpha_2_L group2.M     1 eta_2  4.90486226 0.32015450              
#
#observed statistics:  2221 
#estimated parameters:  8 
#degrees of freedom:  2213 
#-2 log likelihood:  15932.19 
#saturated -2 log likelihood:  NA 
#number of observations:  933 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC: 11506.1851           15948.19                   NA
#BIC:   798.7944           15986.89             15961.49
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-06-02 13:38:11 
#frontend time: 0.158 secs 
#backend time: 0.8010001 secs 
#independent submodels time: 0 secs 
#wall clock time: 0.9590001 secs 
#cpu time: 0.9590001 secs 
#OpenMx version number: 999.0.0-3473 





################# Means, Variances, & Covariance Model ################# 
mg3.math.omx <- mxModel("Multiple Group Growth Model, Means, Covariances, Residuals, Path Specification",
mxModel("group1", type="RAM",
mxData(observed=mathdata_N, type="raw"),
manifestVars=c("math2","math3","math4","math5","math6","math7","math8"), 
latentVars=c("eta_1","eta_2"),

# residual variance paths
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=2, free=TRUE, values=60, labels="th"),

# latent variable variances and covariance paths 
mxPath(from=c("eta_1","eta_2"), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(1,0,1), labels=c("psi_11_N","psi_21_N","psi_22_N")),

# Factor Loadings
mxPath(from="eta_1", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=1),

mxPath(from="eta_2", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

# means and intercepts 
mxPath(from="one", to=c("eta_1","eta_2"),
 arrows=1, free=c(TRUE,TRUE), values=c(100, 15), labels=c("alpha_1_N","alpha_2_N"))

),  # close Normal Birthweight Model   

mxModel("group2", type="RAM",
mxData(observed=mathdata_L, type="raw"),
manifestVars=c("math2","math3","math4","math5","math6","math7","math8"),
latentVars=c("eta_1","eta_2"),

# residual variance paths
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=2, free=TRUE, values=60, labels="th"),

# latent variable variances and covariance paths 
mxPath(from=c("eta_1","eta_2"), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(1,0,1), labels=c("psi_11_L","psi_21_L","psi_22_L")),

# Factor Loadings
mxPath(from="eta_1", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=1),

mxPath(from="eta_2",  to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

# means and intercepts 
mxPath(from="one", to=c("eta_1","eta_2"),
 arrows=1, free=c(TRUE,TRUE), values=c(100, 15), labels=c("alpha_1_L","alpha_2_L"))

), #Close Low Birthweight Model

mxAlgebra(group1.objective + group2.objective, name="mg_objective"),
mxFitFunctionAlgebra("mg_objective")

) #Close Model

mg3.math.fit <- mxRun(mg3.math.omx)

summary(mg3.math.fit)

#free parameters:
#        name   matrix   row   col   Estimate   Std.Error lbound ubound
#1         th group1.S math2 math2 36.2756526  1.89479471              
#2   psi_11_N group1.S eta_1 eta_1 61.0618973  5.87705565              
#3   psi_21_N group1.S eta_1 eta_2  0.2425102  1.21949478              
#4   psi_22_N group1.S eta_2 eta_2  0.6631559  0.33789527              
#5  alpha_1_N group1.M     1 eta_1 35.4850555  0.36480241              
#6  alpha_2_N group1.M     1 eta_2  4.2926990  0.09117542              
#7   psi_11_L group2.S eta_1 eta_1 95.2860831 24.08853057              
#8   psi_21_L group2.S eta_1 eta_2 -3.8020614  4.83159146              
#9   psi_22_L group2.S eta_2 eta_2  1.2973585  1.29363976              
#10 alpha_1_L group2.M     1 eta_1 32.8500392  1.41347271              
#11 alpha_2_L group2.M     1 eta_2  4.8812144  0.34053105              
#
#observed statistics:  2221 
#estimated parameters:  11 
#degrees of freedom:  2210 
#-2 log likelihood:  15929.46 
#saturated -2 log likelihood:  NA 
#number of observations:  933 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC: 11509.4568           15951.46                   NA
#BIC:   816.5814           16004.68             15969.74
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-06-02 13:29:01 
#frontend time: 0.1990001 secs 
#backend time: 1.316 secs 
#independent submodels time: 0 secs 
#wall clock time: 1.515 secs 
#cpu time: 1.515 secs 
#OpenMx version number: 999.0.0-3473 




################# Means, Variances, Covariance, & Residual Variance Model ################# 
mg4.math.omx <- mxModel("Multiple Group Growth Model, Means, Covariances, Residuals, Path Specification",
mxModel("group1", type="RAM",
mxData(observed=mathdata_N, type="raw"),
manifestVars=c("math2","math3","math4","math5","math6","math7","math8"), 
latentVars=c("eta_1","eta_2"),

# residual variance paths
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=2, free=TRUE, values=60, labels="th_N"),

# latent variable variances and covariance paths 
mxPath(from=c("eta_1","eta_2"), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(60,0,5), labels=c("psi_11_N","psi_21_N","psi_22_N")),

# Factor Loadings
mxPath(from="eta_1", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=1),

mxPath(from="eta_2", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

# means and intercepts 
mxPath(from="one", to=c("eta_1","eta_2"),
 arrows=1, free=c(TRUE,TRUE), values=c(100, 15), labels=c("alpha_1_N","alpha_2_N"))

),  # close Normal Birthweight Model   

mxModel("group2", type="RAM",
mxData(observed=mathdata_L, type="raw"),
manifestVars=c("math2","math3","math4","math5","math6","math7","math8"),
latentVars=c("eta_1","eta_2"),

# residual variance paths
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=2, free=TRUE, values=60, labels="th_L"),

# latent variable variances and covariance paths 
mxPath(from=c("eta_1","eta_2"), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(60,0,5), labels=c("psi_11_L","psi_21_L","psi_22_L")),

# Factor Loadings
mxPath(from="eta_1", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=1),

mxPath(from="eta_2",  to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

# means and intercepts 
mxPath(from="one", to=c("eta_1","eta_2"),
 arrows=1, free=c(TRUE,TRUE), values=c(100, 15), labels=c("alpha_1_L","alpha_2_L"))

), #Close Low Birthweight Model

mxAlgebra(group1.objective + group2.objective, name="mg_objective"),
mxFitFunctionAlgebra("mg_objective")

) #Close Model

mg4.math.fit <- mxRun(mg4.math.omx)

summary(mg4.math.fit)

#free parameters:
#        name   matrix   row   col    Estimate   Std.Error lbound ubound
#1       th_N group1.S math2 math2 35.17197839  1.87799257              
#2   psi_11_N group1.S eta_1 eta_1 62.28975865  5.58685757              
#3   psi_21_N group1.S eta_1 eta_2 -0.06320182  1.09799941              
#4   psi_22_N group1.S eta_2 eta_2  0.77373272  0.32227432              
#5  alpha_1_N group1.M     1 eta_1 35.48131985  0.36482093              
#6  alpha_2_N group1.M     1 eta_2  4.29730928  0.09112965              
#7       th_L group2.S math2 math2 48.68704569  8.39569515              
#8   psi_11_L group2.S eta_1 eta_1 79.62008119 25.35468150              
#9   psi_21_L group2.S eta_1 eta_2  0.74648044  5.41729561              
#10  psi_22_L group2.S eta_2 eta_2 -0.15728435  1.45729012              
#11 alpha_1_L group2.M     1 eta_1 32.79999194  1.40680570              
#12 alpha_2_L group2.M     1 eta_2  4.87300524  0.34077354              
#
#observed statistics:  2221 
#estimated parameters:  12 
#degrees of freedom:  2209 
#-2 log likelihood:  15926.11 
#saturated -2 log likelihood:  NA 
#number of observations:  933 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC: 11508.1112           15950.11                   NA
#BIC:   820.0741           16008.17             15970.06
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-06-02 13:18:41 
#frontend time: 0.1599998 secs 
#backend time: 1.612 secs 
#independent submodels time: 0 secs 
#wall clock time: 1.772 secs 
#cpu time: 1.772 secs 
#OpenMx version number: 999.0.0-3473 