# Load the data
nlsy_math_long <- read.table("C:/GRE2016/Data/nlsy_math_long_R.dat")

# Give the variable names
names(nlsy_math_long)<-c('id', 'female', 'lb_wght', 'anti_k1', 'math', 'grade', 'occ', 'age', 'men', 'spring', 'anti')

attach(nlsy_math_long)

library(nlme)

nlsy_math_long$nb_wght <- 1 - (nlsy_math_long$lb_wght)

# Model M1: Means 
mg1.math.nlme<-nlme(math~nb_wght*((beta_1+d_1i)+(beta_2+d_2i)*(grade-2))+lb_wght*((beta_1+d_1i)+(beta_2+d_2i)*(grade-2)),
                      data=nlsy_math_long,
                      fixed=beta_1+beta_2~1,
                      random= d_1i+d_2i~1,
			                group = ~id,
                      start=c(35, 4),
		                  control =(list(returnObject = TRUE)))

summary (mg1.math.nlme)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: math ~ nb_wght * ((beta_1 + d_1i) + (beta_2 + d_2i) * (grade - 2)) +      lb_wght * ((beta_1 + d_1i) + (beta_2 + d_2i) * (grade - 2)) 
# Data: nlsy_math_long 
#       AIC      BIC    logLik
#  15949.39 15983.62 -7968.693
#
#Random effects:
# Formula: list(d_1i ~ 1, d_2i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev    Corr  
#d_1i     8.0350382 d_1i  
#d_2i     0.8558994 -0.026
#Residual 6.0191140       
#
#Fixed effects: beta_1 + beta_2 ~ 1 
#          Value Std.Error   DF  t-value p-value
#beta_1 35.26736 0.3551568 1288 99.30082       0
#beta_2  4.33933 0.0873767 1288 49.66231       0
# Correlation: 
#       beta_1  
#beta_2 -0.532
#
#Standardized Within-Group Residuals:
#         Min           Q1          Med           Q3          Max 
#-3.030277731 -0.525876489  0.001632883  0.540751587  2.542250204 
#
#Number of Observations: 2221
#Number of Groups: 932 




# Model M2: Means 
mg2.math.nlme<-nlme(math~nb_wght*((beta_1_N+d_1i)+(beta_2_N+d_2i)*(grade-2))+lb_wght*((beta_1_L+d_1i)+(beta_2_L+d_2i)*(grade-2)),
                      data=nlsy_math_long,
                      fixed=beta_1_N+beta_2_N+beta_1_L+beta_2_L~1,
                      random= d_1i+d_2i~1,
			                group = ~id,
                      start=c(35, 4, 35, 4),
		                  control =(list(returnObject = TRUE)))

summary (mg2.math.nlme)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: math ~ nb_wght * ((beta_1_N + d_1i) + (beta_2_N + d_2i) * (grade -      2)) + lb_wght * ((beta_1_L + d_1i) + (beta_2_L + d_2i) * (grade -      2)) 
# Data: nlsy_math_long 
#       AIC      BIC    logLik
#  15948.19 15993.83 -7966.093
#
#Random effects:
# Formula: list(d_1i ~ 1, d_2i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev    Corr  
#d_1i     7.9814759 d_1i  
#d_2i     0.8360323 -0.005
#Residual 6.0266663       
#
#Fixed effects: beta_1_N + beta_2_N + beta_1_L + beta_2_L ~ 1 
#            Value Std.Error   DF  t-value p-value
#beta_1_N 35.48755 0.3693085  931 96.09190       0
#beta_2_N  4.29228 0.0906143 1287 47.36865       0
#beta_1_L 32.73331 1.2445562 1287 26.30119       0
#beta_2_L  4.90490 0.3201714 1287 15.31962       0
# Correlation: 
#       beta_1_N beta_2_N beta_1_L
#beta_2_N -0.530              
#beta_1_L  0.000  0.000       
#beta_2_L  0.000  0.000 -0.529
#
#Standardized Within-Group Residuals:
#         Min           Q1          Med           Q3          Max 
#-3.063587308 -0.528511582 -0.003018735  0.536375325  2.542164473 
#
#Number of Observations: 2221
#Number of Groups: 932 



# Model M3: Means and Covariances
mg3.math.nlme<-nlme(math~nb_wght*((beta_1_N+d_1i_N)+(beta_2_N+d_2i_N)*(grade-2))+lb_wght*((beta_1_L+d_1i_L)+(beta_2_L+d_2i_L)*(grade-2)),
                      data=nlsy_math_long,
                      fixed=beta_1_N+beta_2_N+beta_1_L+beta_2_L~1,
                      random= pdBlocked(list(d_1i_N+d_2i_N~1, d_1i_L+d_2i_L~1)),
			                group = ~id,
                      start=c(35, 4, 35, 4),
		                  control =(list(returnObject = TRUE)))

summary (mg3.math.nlme)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: math ~ nb_wght * ((beta_1_N + d_1i_N) + (beta_2_N + d_2i_N) * (grade -      2)) + lb_wght * ((beta_1_L + d_1i_L) + (beta_2_L + d_2i_L) *      (grade - 2)) 
# Data: nlsy_math_long 
#       AIC      BIC    logLik
#  15951.46 16014.22 -7964.728
#
#Random effects:
# Composite Structure: Blocked
#
# Block 1: d_1i_N, d_2i_N
# Formula: list(d_1i_N ~ 1, d_2i_N ~ 1)
# Level: id
# Structure: General positive-definite
#       StdDev    Corr  
#d_1i_N 7.8142181 d_1i_N
#d_2i_N 0.8143424 0.038 
#
# Block 2: d_1i_L, d_2i_L
# Formula: list(d_1i_L ~ 1, d_2i_L ~ 1)
# Level: id
# Structure: General positive-definite
#         StdDev   Corr  
#d_1i_L   9.761396 d_1i_L
#d_2i_L   1.139022 -0.342
#Residual 6.022932       
#
#Fixed effects: beta_1_N + beta_2_N + beta_1_L + beta_2_L ~ 1 
#            Value Std.Error   DF  t-value p-value
#beta_1_N 35.48504 0.3647143 1287 97.29542       0
#beta_2_N  4.29270 0.0901961 1287 47.59293       0
#beta_1_L 32.85003 1.4112477  931 23.27729       0
#beta_2_L  4.88120 0.3377600 1287 14.45169       0
# Correlation: 
#       beta_1_N beta_2_N beta_1_L
#beta_2_N -0.527              
#beta_1_L  0.000  0.000       
#beta_2_L  0.000  0.000 -0.564
#
#Standardized Within-Group Residuals:
#         Min           Q1          Med           Q3          Max 
#-3.095521050 -0.531376590 -0.000980768  0.535246213  2.542557118 
#
#Number of Observations: 2221
#Number of Groups: 932 




# Model M4: Means, Covariances, and Residual Variance
mg4.math.nlme<-nlme(math~nb_wght*((beta_1_N+d_1i_N)+(beta_2_N+d_2i_N)*(grade-2))+lb_wght*((beta_1_L+d_1i_L)+(beta_2_L+d_2i_L)*(grade-2)),
                      data=nlsy_math_long,
                      fixed=beta_1_N+beta_2_N+beta_1_L+beta_2_L~1,
                      random=pdBlocked(list(d_1i_N+d_2i_N~1, d_1i_L+d_2i_L~1)),
			          group = ~id,
                      start=c(35, 4, 35, 4),
		              weights = varIdent(form =~ 1|factor(lb_wght)),
                      control =(list(returnObject = TRUE)))

summary (mg4.math.nlme)

#Warning messages:
#1: In nlme.formula(math ~ nb_wght * ((beta_1_N + d_1i_N) + (beta_2_N +  :
#  step halving factor reduced below minimum in PNLS step
#2: In nlme.formula(math ~ nb_wght * ((beta_1_N + d_1i_N) + (beta_2_N +  :
#  step halving factor reduced below minimum in PNLS step


#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: math ~ nb_wght * ((beta_1_N + d_1i_N) + (beta_2_N + d_2i_N) * (grade -      2)) + lb_wght * ((beta_1_L + d_1i_L) + (beta_2_L + d_2i_L) *      (grade - 2)) 
# Data: nlsy_math_long 
#       AIC      BIC    logLik
#  15950.12 16018.59 -7963.061
#
#Random effects:
# Composite Structure: Blocked#
#
# Block 1: d_1i_N, d_2i_N
# Formula: list(d_1i_N ~ 1, d_2i_N ~ 1)
# Level: id
# Structure: General positive-definite
#       StdDev   Corr  
#d_1i_N 7.892198 d_1i_N
#d_2i_N 0.879575 -0.009
#
# Block 2: d_1i_L, d_2i_L
# Formula: list(d_1i_L ~ 1, d_2i_L ~ 1)
# Level: id
# Structure: General positive-definite
#         StdDev     Corr  
#d_1i_L   8.99419312 d_1i_L
#d_2i_L   0.02845102 0.988 
#Residual 5.93062214       
#
#Variance function:
# Structure: Different standard deviations per stratum
# Formula: ~1 | factor(lb_wght) 
# Parameter estimates:
#       0        1 
#1.000000 1.170217 
#Fixed effects: beta_1_N + beta_2_N + beta_1_L + beta_2_L ~ 1 
#            Value Std.Error   DF  t-value p-value
#beta_1_N 35.48131 0.3647426  931 97.27766       0
#beta_2_N  4.29731 0.0901739 1287 47.65583       0
#beta_1_L 32.79781 1.4062570 1287 23.32277       0
#beta_2_L  4.87806 0.3382247 1287 14.42253       0
# Correlation: 
#       beta_1_N beta_2_N beta_1_L
#beta_2_N -0.527              
#beta_1_L  0.000  0.000       
#beta_2_L  0.000  0.000 -0.549
#
#Standardized Within-Group Residuals:
#         Min           Q1          Med           Q3          Max 
#-3.080667705 -0.533467874 -0.002345949  0.537432288  2.566341980 
#
#Number of Observations: 2221
#Number of Groups: 932 







#### Alternative Approach ####

# Multiple Group Model M1: Invariance
mg1.math.nlme <- nlme(math~(beta_1+d_1i)+(beta_2+d_2i)*(grade-2),
                      data=nlsy_math_long,
                      fixed=beta_1+beta_2~1,
                      random=d_1i+d_2i~1,
                      groups = ~ id, 
                      start=c(35, 4))

summary (mg1.math.nlme)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: math ~ (beta_1 + d_1i) + (beta_2 + d_2i) * (grade - 2) 
# Data: nlsy_math_long 
#       AIC      BIC    logLik
#  15949.39 15983.62 -7968.693
#
#Random effects:
# Formula: list(d_1i ~ 1, d_2i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev    Corr  
#d_1i     8.0350382 d_1i  
#d_2i     0.8558994 -0.026
#Residual 6.0191140       
#
#Fixed effects: beta_1 + beta_2 ~ 1 
#          Value Std.Error   DF  t-value p-value
#beta_1 35.26736 0.3551568 1288 99.30082       0
#beta_2  4.33933 0.0873767 1288 49.66231       0
# Correlation: 
#     beta_1  
#beta_2 -0.532
#
#Standardized Within-Group Residuals:
#         Min           Q1          Med           Q3          Max 
#-3.030277731 -0.525876489  0.001632883  0.540751587  2.542250204 
#
#Number of Observations: 2221
#Number of Groups: 932 




# Multiple Group Model M2: Means
mg2.math.nlme <- nlme(math~(beta_1+d_1i)+(beta_2+d_2i)*(grade-2),
                      data=nlsy_math_long,
                      fixed=beta_1+beta_2 ~ factor(lb_wght)-1,
                      random=d_1i+d_2i~1,
                      groups = ~ id, 
                      start=c(35, 35, 4, 4))
                      
summary (mg2.math.nlme)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: math ~ (beta_1 + d_1i) + (beta_2 + d_2i) * (grade - 2) 
# Data: nlsy_math_long 
#       AIC      BIC    logLik
#  15948.19 15993.83 -7966.093
#
#Random effects:
# Formula: list(d_1i ~ 1, d_2i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev   Corr  
#d_1i     7.981476 d_1i  
#d_2i     0.836031 -0.005
#Residual 6.026667       
#
#Fixed effects: beta_1 + beta_2 ~ factor(lb_wght) - 1 
#                           Value Std.Error   DF  t-value p-value
#beta_1.factor(lb_wght)0 35.48755 0.3693085  931 96.09189       0
#beta_1.factor(lb_wght)1 32.73331 1.2445563 1287 26.30119       0
#beta_2.factor(lb_wght)0  4.29228 0.0906144 1287 47.36864       0
#beta_2.factor(lb_wght)1  4.90490 0.3201713 1287 15.31962       0
# Correlation: 
#                      m_1.(_)0 m_1.(_)1 m_2.(_)0
#beta_1.factor(lb_wght)1  0.000                    
#beta_2.factor(lb_wght)0 -0.530    0.000           
#beta_2.factor(lb_wght)1  0.000   -0.529    0.000  
#
#Standardized Within-Group Residuals:
#         Min           Q1          Med           Q3          Max 
#-3.063587024 -0.528511536 -0.003018735  0.536375276  2.542164239 
#
#Number of Observations: 2221
#Number of Groups: 932 




# Multiple Group Model M3: Means + Covariances
# Note that too many covariances are estimated in this model, but estimates are accurate
mg3.math.nlme <- nlme(math~(beta_1+d_1i)+(beta_2+d_2i)*(grade-2),
                      data=nlsy_math_long,
                      fixed=beta_1+beta_2 ~ factor(lb_wght)-1,
                      random=d_1i+d_2i ~ factor(lb_wght)-1,
                      groups = ~ id,
                      start=c(35, 35, 4, 4))

summary (mg3.math.nlme)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: math ~ nb_wght * ((beta_1_N + d_1i_N) + (beta_2_N + d_2i_N) * (grade -      2)) + lb_wght * ((beta_1_L + d_1i_L) + (beta_2_L + d_2i_L) *      (grade - 2)) 
# Data: nlsy_math_long 
#       AIC      BIC    logLik
#  15951.46 16014.22 -7964.728
#
#Random effects:
# Composite Structure: Blocked
#
# Block 1: d_1i_N, d_2i_N
# Formula: list(d_1i_N ~ 1, d_2i_N ~ 1)
# Level: id
# Structure: General positive-definite
#       StdDev    Corr  
#d_1i_N 7.8142181 d_1i_N
#d_2i_N 0.8143424 0.038 
#
# Block 2: d_1i_L, d_2i_L
# Formula: list(d_1i_L ~ 1, d_2i_L ~ 1)
# Level: id
# Structure: General positive-definite
#         StdDev   Corr  
#d_1i_L   9.761396 d_1i_L
#d_2i_L   1.139022 -0.342
#Residual 6.022932       
#
#Fixed effects: beta_1_N + beta_2_N + beta_1_L + beta_2_L ~ 1 
#            Value Std.Error   DF  t-value p-value
#beta_1_N 35.48504 0.3647143 1287 97.29542       0
#beta_2_N  4.29270 0.0901961 1287 47.59293       0
#beta_1_L 32.85003 1.4112477  931 23.27729       0
#beta_2_L  4.88120 0.3377600 1287 14.45169       0
# Correlation: 
#       beta_1_N beta_2_N beta_1_L
#beta_2_N -0.527              
#beta_1_L  0.000  0.000       
#beta_2_L  0.000  0.000 -0.564
#
#Standardized Within-Group Residuals:
#         Min           Q1          Med           Q3          Max 
#-3.095521050 -0.531376590 -0.000980768  0.535246213  2.542557118 
#
#Number of Observations: 2221
#Number of Groups: 932 




#Multiple Group Model M4: Means, Covariances, Residuals
mg4.math.nlme <- nlme(math~(beta_1+d_1i)+(beta_2+d_2i)*(grade-2),
                      data=nlsy_math_long,
                      fixed=beta_1+beta_2 ~ factor(lb_wght)-1,
                      random=d_1i+d_2i ~ factor(lb_wght)-1,
                      groups = ~ id,
                      start=c(35, 35, 4, 4),
                      weights = varIdent(form =~ 1|factor(lb_wght))
                      )

summary (mg4.math.nlme)


#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: math ~ (beta_1 + d_1i) + (beta_2 + d_2i) * (grade - 2) 
# Data: nlsy_math_long 
#       AIC      BIC    logLik
#  15958.13 16049.42 -7963.063
#
#Random effects:
# Formula: list(d_1i ~ factor(lb_wght) - 1, d_2i ~ factor(lb_wght) - 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#                         StdDev    Corr                    
#d_1i.factor(lb_wght)0 7.8922113 d_1.(_)0 d_1.(_)1 d_2.(_
#d_1i.factor(lb_wght)1 9.0222996  0.016                  
#d_2i.factor(lb_wght)0 0.8795845 -0.009   -0.004         
#d_2i.factor(lb_wght)1 0.1464057  0.001    0.090    0.001
#Residual              5.9306101                         
#
#Variance function:
# Structure: Different standard deviations per stratum
# Formula: ~1 | factor(lb_wght) 
# Parameter estimates:
#       0        1 
#1.000000 1.169336 
#Fixed effects: beta_1 + beta_2 ~ factor(lb_wght) - 1 
#                         Value Std.Error   DF  t-value p-value
#beta_1.factor(lb_wght)0 35.48131 0.3647428 1286 97.27761       0
#beta_1.factor(lb_wght)1 32.79999 1.4084712 1286 23.28765       0
#beta_2.factor(lb_wght)0  4.29731 0.0901739 1286 47.65582       0
#beta_2.factor(lb_wght)1  4.87824 0.3385636 1286 14.40862       0
# Correlation: 
#                        m_1.(_)0 m_1.(_)1 m_2.(_)0
#beta_1.factor(lb_wght)1  0.000                    
#beta_2.factor(lb_wght)0 -0.527    0.000           
#beta_2.factor(lb_wght)1  0.000   -0.551    0.000  
#
#Standardized Within-Group Residuals:
#         Min           Q1          Med           Q3          Max 
#-3.080664775 -0.533468176 -0.002345252  0.538209085  2.566344773 
#
#Number of Observations: 2221
#Number of Groups: 932 



### Yet another approach to fitting Model M4 - fitting separate models for each group

#Multiple Group Model M4: Means, Covariances, Residuals
mg4.math.nlme0 <- nlme(math~(beta_1+d_1i)+(beta_2+d_2i)*(grade-2),
                      data=nlsy_math_long,
                      fixed=beta_1+beta_2 ~ 1,
                      random=d_1i+d_2i ~ 1,
                      groups = ~ id,
                      start=c(35, 4),
                      subset = lb_wght == 0)

summary (mg4.math.nlme0)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: math ~ (beta_1 + d_1i) + (beta_2 + d_2i) * (grade - 2) 
# Data: nlsy_math_long 
#  Subset: lb_wght == 0 
#       AIC      BIC    logLik
#  14630.33 14664.07 -7309.167
#
#Random effects:
# Formula: list(d_1i ~ 1, d_2i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev    Corr  
#d_1i     7.8922007 d_1i  
#d_2i     0.8795762 -0.009
#Residual 5.9306223       
#
#Fixed effects: beta_1 + beta_2 ~ 1 
#          Value Std.Error   DF  t-value p-value
#beta_1 35.48131 0.3645926 1186 97.31769       0
#beta_2  4.29731 0.0901368 1186 47.67544       0
# Correlation: 
#       beta_1  
#beta_2 -0.527
#
#Standardized Within-Group Residuals:
#         Min           Q1          Med           Q3          Max 
#-3.080666831 -0.532927926 -0.004167321  0.537387007  2.566341518 
#
#Number of Observations: 2044
#Number of Groups: 857 


mg4.math.nlme1 <- nlme(math~(beta_1+d_1i)+(beta_2+d_2i)*(grade-2),
                      data=nlsy_math_long,
                      fixed=beta_1+beta_2 ~ 1,
                      random=d_1i+d_2i ~ 1,
                      groups = ~ id,
                      start=c(35, 4),
                      subset = lb_wght == 1)

summary (mg4.math.nlme1)


#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: math ~ (beta_1 + d_1i) + (beta_2 + d_2i) * (grade - 2) 
# Data: nlsy_math_long 
#  Subset: lb_wght == 1 
#       AIC      BIC    logLik
#  1319.796 1338.853 -653.8981
#
#Random effects:
# Formula: list(d_1i ~ 1, d_2i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev     Corr  
#d_1i     9.06336910 d_1i  
#d_2i     0.00153797 -0.002
#Residual 6.93872948       
#
#Fixed effects: beta_1 + beta_2 ~ 1 
#          Value Std.Error  DF  t-value p-value
#beta_1 32.80686 1.4188779 101 23.12169       0
#beta_2  4.87633 0.3394461 101 14.36554       0
# Correlation: 
#       beta_1  
#beta_2 -0.554
#
#Standardized Within-Group Residuals:
#          Min            Q1           Med            Q3           Max 
#-2.2252990363 -0.5938630861 -0.0004514727  0.5441954942  2.1876330937 
#
#Number of Observations: 177
#Number of Groups: 75 
