*Reading Data into SAS;
DATA nlsy_math_long;
	INFILE 'C:\GRE2016\Data\nlsy_math_long_R.dat';
	INPUT 
		id	female 		lb_wght		anti_k1

		math 	grade		occ			age
		men		spring		anti;
RUN;


*Fitting Multiple Group Linear Growth Model with lb_wght as Grouping Variable in SAS using NLMIXED;

*Model M1: Invariance;
PROC NLMIXED DATA = nlsy_math_long;
	IF lb_wght = 0 THEN 
		b_1i = beta_1 + d_1i_N;
	IF lb_wght = 0 THEN 
		b_2i = beta_2 + d_2i_N;

	IF lb_wght = 1 THEN 
		b_1i = beta_1 + d_1i_L;
	IF lb_wght = 1 THEN 
		b_2i = beta_2 + d_2i_L;

	traject = b_1i + b_2i * (grade - 2);

	MODEL math ~ NORMAL(traject, v_u);
	RANDOM d_1i_N d_2i_N d_1i_L d_2i_L 
						~ NORMAL([0,0,0,0], [v_1,
									  		c_21, v_2,
                                                 0,     0, v_1,
									  		     0,     0, c_21, v_2])
	SUBJECT = id;
	PARMS	
		beta_1 = 40	beta_2 = 5
	
		v_1 = 70	v_2 = 1	    c_21 = 0

		v_u = 40;
RUN;

/*
                                           Fit Statistics

                              -2 Log Likelihood                  15937
                              AIC (smaller is better)            15949
                              AICC (smaller is better)           15949
                              BIC (smaller is better)            15978


                                        Parameter Estimates

                         Standard
    Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

    beta_1      35.2673    0.3554   928    99.23    <.0001    0.05   34.5698   35.9648   -0.0002
    beta_2       4.3393   0.08831   928    49.14    <.0001    0.05    4.1660    4.5126  -0.00036
    v_1         64.5617    5.6594   928    11.41    <.0001    0.05   53.4550   75.6684  5.631E-6
    v_2          0.7325    0.3273   928     2.24    0.0254    0.05   0.09024    1.3749  0.000022
    c_21        -0.1814    1.1501   928    -0.16    0.8747    0.05   -2.4384    2.0756  0.000081
    v_u         36.2298    1.8665   928    19.41    <.0001    0.05   32.5666   39.8929  -0.00001
*/

*Model M2: Means;
PROC NLMIXED DATA = nlsy_math_long;
	IF lb_wght = 0 THEN 
		b_1i = beta_1_N + d_1i_N;
	IF lb_wght = 0 THEN 
		b_2i = beta_2_N + d_2i_N;

	IF lb_wght = 1 THEN 
		b_1i = beta_1_L + d_1i_L;
	IF lb_wght = 1 THEN 
		b_2i = beta_2_L + d_2i_L;

	traject = b_1i + b_2i * (grade - 2);

	MODEL math ~ NORMAL(traject, v_u);
	RANDOM d_1i_N d_2i_N d_1i_L d_2i_L 
						~ NORMAL([0,0,0,0], [v_1,
									  		c_21, v_2,
                                                 0,     0, v_1,
									  		     0,     0, c_21, v_2])
	SUBJECT = id;
	PARMS	
		beta_1_N = 40	beta_2_N = 5
		beta_1_L = 40	beta_2_L = 5
	
		v_1 = 70	v_2 = 1	     c_21 = 0

		v_u = 40;
RUN;

/*
                                           Fit Statistics

                              -2 Log Likelihood                  15932
                              AIC (smaller is better)            15948
                              AICC (smaller is better)           15948
                              BIC (smaller is better)            15987


                                        Parameter Estimates

                         Standard
    Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

    beta_1_N    35.4876    0.3694   928    96.08    <.0001    0.05   34.7627   36.2124  -5.34E-6
    beta_2_N     4.2923   0.09152   928    46.90    <.0001    0.05    4.1127    4.4719  8.719E-6
    beta_1_L    32.7333    1.2439   928    26.31    <.0001    0.05   30.2921   35.1746  6.063E-6
    beta_2_L     4.9049    0.3202   928    15.32    <.0001    0.05    4.2766    5.5332  4.902E-6
    v_1         63.7039    5.6369   928    11.30    <.0001    0.05   52.6414   74.7664  -1.56E-6
    v_2          0.6989    0.3255   928     2.15    0.0320    0.05   0.06017    1.3377  -0.00001
    c_21       -0.03526    1.1443   928    -0.03    0.9754    0.05   -2.2810    2.2105  -0.00002
    v_u         36.3207    1.8710   928    19.41    <.0001    0.05   32.6489   39.9926  7.132E-6
*/

*Model M3: Means, Variances, Covariances;
PROC NLMIXED DATA = nlsy_math_long;
	IF lb_wght = 0 THEN 
		b_1i = beta_1_N + d_1i_N;
	IF lb_wght = 0 THEN 
		b_2i = beta_2_N + d_2i_N;

	IF lb_wght = 1 THEN 
		b_1i = beta_1_L + d_1i_L;
	IF lb_wght = 1 THEN 
		b_2i = beta_2_L + d_2i_L;

	traject = b_1i + b_2i * (grade - 2);

	MODEL math ~ NORMAL(traject, v_u);
	RANDOM d_1i_N d_2i_N d_1i_L d_2i_L 
						~ NORMAL([0,0,0,0], [v_1_N,
									  		c_21_N, v_2_N,
                                                 0,     0, v_1_L,
									  		     0,     0, c_21_L, v_2_L])
	SUBJECT = id;
	PARMS	
		beta_1_N = 40	beta_2_N = 5
		beta_1_L = 40	beta_2_L = 5
	
		v_1_N = 70	v_2_N = 1	c_21_N = 0
		v_1_L = 70	v_2_L = 1	c_21_L = 0

		v_u = 40;
RUN;

/*
                                           Fit Statistics

                              -2 Log Likelihood                  15929
                              AIC (smaller is better)            15951
                              AICC (smaller is better)           15952
                              BIC (smaller is better)            16005


                                        Parameter Estimates

                         Standard
    Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

    beta_1_N    35.4851    0.3648   928    97.27    <.0001    0.05   34.7691   36.2010  0.000233
    beta_2_N     4.2927   0.09116   928    47.09    <.0001    0.05    4.1138    4.4716  -0.00041
    beta_1_L    32.8501    1.4135   928    23.24    <.0001    0.05   30.0762   35.6241  0.000063
    beta_2_L     4.8812    0.3406   928    14.33    <.0001    0.05    4.2128    5.5496  0.000169
    v_1_N       61.0619    5.6923   928    10.73    <.0001    0.05   49.8906   72.2333  0.000014
    v_2_N        0.6631    0.3262   928     2.03    0.0424    0.05   0.02288    1.3034  0.000071
    c_21_N       0.2426    1.1466   928     0.21    0.8325    0.05   -2.0077    2.4929  0.000054
    v_1_L       95.2818   24.2201   928     3.93    <.0001    0.05   47.7493    142.81  -3.73E-6
    v_2_L        1.2971    1.3149   928     0.99    0.3242    0.05   -1.2835    3.8776  -0.00012
    c_21_L      -3.8010    4.9120   928    -0.77    0.4392    0.05  -13.4410    5.8389  -0.00002
    v_u         36.2759    1.8697   928    19.40    <.0001    0.05   32.6065   39.9452  0.000013
*/

*Model M4: Means, Variances, Covariances, Residual Variances;
PROC NLMIXED DATA = nlsy_math_long;
	IF lb_wght = 0 THEN b_1i = beta_1_N + d_1i_N; 
	IF lb_wght = 0 THEN b_2i = beta_2_N + d_2i_N;

	IF lb_wght = 1 THEN b_1i = beta_1_L + d_1i_L;
	IF lb_wght = 1 THEN b_2i = beta_2_L + d_2i_L;

	IF lb_wght = 0 THEN v_u = v_u_N;
	IF lb_wght = 1 THEN v_u = v_u_L;

	traject = b_1i + b_2i * (grade - 2);

	MODEL math ~ NORMAL(traject, v_u); 
	RANDOM d_1i_N d_2i_N d_1i_L d_2i_L ~ 
                     NORMAL([0,0,0,0], [v_1_N,
                                       c_21_N, v_2_N,
                                            0,     0,  v_1_L,
                                            0,     0, c_21_L, v_2_L])
	SUBJECT = id;
	PARMS
        beta_1_N = 40   beta_2_N = 5    beta_1_L = 40	beta_2_L = 5 
		v_1_N = 70	v_2_N = 1	 c_21_N = 0 	
        v_1_L = 70	v_2_L = 1	 c_21_L = 0
		v_u_N = 40    v_u_L = 40;
RUN; 
/*
                                           Fit Statistics

                              -2 Log Likelihood                  15926
                              AIC (smaller is better)            15950
                              AICC (smaller is better)           15950
                              BIC (smaller is better)            16008


                                        Parameter Estimates

                         Standard
    Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

    beta_1_N    35.4839    0.3642   928    97.43    <.0001    0.05   34.7691   36.1987  0.017754
    beta_2_N     4.2966   0.09115   928    47.14    <.0001    0.05    4.1178    4.4755  -0.04645
    beta_1_L    32.7415    1.3732   928    23.84    <.0001    0.05   30.0465   35.4365  -0.00461
    beta_2_L     4.8967    0.3460   928    14.15    <.0001    0.05    4.2177    5.5756  0.015453
    v_1_N       61.8336    5.6946   928    10.86    <.0001    0.05   50.6577   73.0094   -0.0104
    v_2_N        0.7644    0.3342   928     2.29    0.0224    0.05    0.1085    1.4203  0.122723
    c_21_N      0.01000    1.1565   928     0.01    0.9931    0.05   -2.2596    2.2796  0.033286
    v_1_L       72.5195   23.4971   928     3.09    0.0021    0.05   26.4060    118.63  -0.02211
    v_2_L       0.01330    1.5338   928     0.01    0.9931    0.05   -2.9969    3.0235  0.098766
    c_21_L       0.7787    5.4204   928     0.14    0.8858    0.05   -9.8590   11.4164  -0.04401
    v_u_N       35.2548    1.9112   928    18.45    <.0001    0.05   31.5041   39.0056  0.013957
    v_u_L       48.5251    8.4233   928     5.76    <.0001    0.05   31.9941   65.0561  -0.00345
*/

